///**
// * 3L was made for YOU to help you create awesome websites
// * and fill the Internet with excessive amount of Love! ♥
// *
// * Made by Mateusz Kocz   -> http://radiatingstar.com
// *
// * Watch 3L on Github:    -> https://github.com/mateuszkocz/3l
// * Submit an issue:       -> https://github.com/mateuszkocz/3l/issues?state=open
// *
// * Download the newest version at http://mateuszkocz.github.com/3l
// *
// * MIT Licensed
// * Includes normalize.css, reset.css and some code from HTML5 Boilerplate.
// * For the licencess refer to the LICENCES.md.
// *
// * Version: 1.4.4 (2013.06.02)
// *
// * To compile all of the code you need to use a compiler
// * that supports JavaScript code and guarder mixins.
// * 
// * Recommended compilers:
// * * Winless http://winless.org/
// * * Prepross http://alphapixels.com/prepros/
// * * Codekit http://incident57.com/codekit/
// **/

///**
// * SEO & HTML Debugging
// *
// * A useful supporting class that will help you a bit with your SEO
// * and usability of your website.
// *
// * @ABOUT
// * Creating a website is a serious business, but sometimes you may
// * forget about some important details. This class will help you.
// * It will let you know if you haven't put an alt attribute on an image
// * or kept that attribute empty, when you haven't typed an URL in an anchor or when
// * an anchor link has rel=nofollow attribute. It will also show you an alert
// * message if you don't have a <title> tag and description or left them empty(*).
// *
// * If any of those errors happens, an element will get a red outline
// * border drawing your attention and asking for some love
// * or you will get a message with the same purpose.
// *
// *    (*) The <title> reminder can alert you even if you have a <title> tag.
// *        This happens if you have a <link> tag (usually used for style sheets)
// *        before <title> in your HTML. Just change the order of <link> and
// *        <title> and you're cool.
// *
// * @USAGE
// * Just place .seo-helper() class in the stylesheet.
// *
// * @EXAMPLE
// * .seo-helper()
// *
// * That way every element on your page will be affected
// * by this rule. You can also place it in any other container
// * class element in your code (body, div.wrapper, article and so on)
// * if you don't want for some reasons to check for bugs on whole page
// * but in a single area.
// *
// * Caution! It's only a developement support. Do not forget to delete
// * this class before finishing yourproject and deploying to production,
// * unless you want to keep it in continuous project (like blog)
// * and debug every new content on your site. But then you might consider
// * restyling this class to be a bit more eye-pleasing.
// *
// * This may not work in every browser, but since you are a web 
// * developer/designer you're probably already using a bleeding
// * edge nightly alpha back-door version of browser, so no problem!
// * It's probably the only class you don't need to care about how
// * your visitors will see it!
// *
// * @BROSERS SUPPORT
// * (unknown / not important)
// *
// * @LINKS
// * * http://radiatingstar.com/how-to-improve-seo-with-css
// *
// * @ASIDE
// * Why img:not([alt]), img[alt=""] and img[alt^=" "]?
// * First targets images that don't have an alt attribute, second targets
// * ones that have this attribute but it's left empty (probably left by
// * automatic completion of html editor), third is just in case - it
// * might have been left by html editor or manually "to do it later" (clearly
// * a space at the beginning of an alt can't be anything good).
// *
// * Why a[href=""]?
// * If you write an anchor text and leave href to copyPaste URL later,
// * it will reminding you about that.
// *
// * Why a[rel="nofollow"]?
// * For some reasons links on your site may have this attribute and this
// * may generate a huge SEO problem. Better fix it ASAP. (On the other hand
// * nofollow links might be useful in some situations. See here:
// * -> http://en.wikipedia.org/wiki/Nofollow#Control_internal_PageRank_flow)
// *
// * Why div:empty, span:empty, li:empty, p:empty, td:empty, th:empty?
// * It'll just check if you have some redundancy (empty elements) in you code.
// *
// **/
 
.seo-helper () {
	img:not([alt]), img[alt=""], img[alt^=" "],
	a[href=""], a[href^=" "], a[href="#"], a[rel*="nofollow"],
	div:empty, span:empty, li:empty, p:empty, td:empty, th:empty,
	*[title=""], *[class=""], *[id=""] {
		outline: 2px solid red !important;
		outline-offset: 3px !important;
		}
	head, title:empty, link, meta {display: block;}
	title:empty:before {content: "You've left the <title> empty!"}
	link:before {content: "You don't have a <title>!"}
	title ~ link {display: none;}
	meta[name="description"][content=""]:before, meta[name="description"][content=" "]:before {content: "You've left description empty!";}
	}

///**
// * Normalize or reset CSS
// * 
// * @USAGE
// * Just write chosen method in your stylesheet.
// *
// * @EXAMLPES
// * /* In CSS */
// * .normalize()
// **/

.normalize(){article,aside,details,figcaption,figure,footer,header,hgroup,main,nav,section,summary{display:block}audio,canvas,video{display:inline-block}audio:not([controls]){display:none;height:0}[hidden]{display:none}html{font-family:sans-serif;-ms-text-size-adjust:100%;-webkit-text-size-adjust:100%}body{margin:0}a:focus{outline:thin dotted}a:active,a:hover{outline:0}h1{font-size:2em;margin:.67em 0}abbr[title]{border-bottom:1px dotted}b,strong{font-weight:bold}dfn{font-style:italic}hr{-moz-box-sizing:content-box;box-sizing:content-box;height:0}mark{background:#ff0;color:#000}code,kbd,pre,samp{font-family:monospace,serif;font-size:1em}pre{white-space:pre-wrap}q{quotes:"\201C" "\201D" "\2018" "\2019"}small{font-size:80%}sub,sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}sup{top:-0.5em}sub{bottom:-0.25em}img{border:0}svg:not(:root){overflow:hidden}figure{margin:0}fieldset{border:1px solid silver;margin:0 2px;padding:.35em .625em .75em}legend{border:0;padding:0}button,input,select,textarea{font-family:inherit;font-size:100%;margin:0}button,input{line-height:normal}button,select{text-transform:none}button,html input[type="button"],input[type="reset"],input[type="submit"]{-webkit-appearance:button;cursor:pointer}button[disabled],html input[disabled]{cursor:default}input[type="checkbox"],input[type="radio"]{box-sizing:border-box;padding:0}input[type="search"]{-webkit-appearance:textfield;-moz-box-sizing:content-box;-webkit-box-sizing:content-box;box-sizing:content-box}input[type="search"]::-webkit-search-cancel-button,input[type="search"]::-webkit-search-decoration{-webkit-appearance:none}button::-moz-focus-inner,input::-moz-focus-inner{border:0;padding:0}textarea{overflow:auto;vertical-align:top}table{border-collapse:collapse;border-spacing:0}}
.reset() {html,body,div,span,applet,object,iframe,h1,h2,h3,h4,h5,h6,p,blockquote,pre,a,abbr,acronym,address,big,cite,code,del,dfn,em,img,ins,kbd,q,s,samp,small,strike,strong,sub,sup,tt,var,b,u,i,center,dl,dt,dd,ol,ul,li,fieldset,form,label,legend,table,caption,tbody,tfoot,thead,tr,th,td,article,aside,canvas,details,embed,figure,figcaption,footer,header,hgroup,menu,nav,output,ruby,section,summary,time,mark,audio,video{margin:0;padding:0;border:0;font-size:100%;font:inherit;vertical-align:baseline}article,aside,details,figcaption,figure,footer,header,hgroup,menu,nav,section{display:block}body{line-height:1}ol,ul{list-style:none}blockquote,q{quotes:none}blockquote:before,blockquote:after,q:before,q:after{content:'';content:none}table{border-collapse:collapse;border-spacing:0}}

///**
// * HTML5 Boiler Plate's default stylesheets.
// * 
// * @USAGE
// * Just write chosen method in your CSS.
// *
// * @EXAMLPES
// * /* In CSS */
// * .h5bp()
// **/

.h5bp() {html,button,input,select,textarea{color:#222}body{font-size:1em;line-height:1.4}::-moz-selection{background:#b3d4fc;text-shadow:none}::selection{background:#b3d4fc;text-shadow:none}hr{display:block;height:1px;border:0;border-top:1px solid #ccc;margin:1em 0;padding:0}img{vertical-align:middle}fieldset{border:0;margin:0;padding:0}textarea{resize:vertical}.chromeframe{margin:.2em 0;background:#ccc;color:#000;padding:.2em 0}}
.h5bp-print() {@media print{*{background:transparent !important;color:#000 !important;box-shadow:none !important;text-shadow:none !important;} a,a:visited{text-decoration:underline;} a[href]:after{content:" (" attr(href) ")";} abbr[title]:after{content:" (" attr(title) ")";} .ir a:after,a[href^="javascript:"]:after,a[href^="#"]:after{content:"";} pre,blockquote{border:1px solid #999;page-break-inside:avoid;} thead{display:table-header-group;} tr,img{page-break-inside:avoid;} img{max-width:100% !important;} @page {margin:0.5cm;}p,h2,h3{orphans:3;widows:3;} h2,h3{page-break-after:avoid;}}.przerwa{color:#ff0000;}}
	
///**
// * Helper classes from HTML5 Boilerplate
// * 
// * Classes below help you create a better user experience for both users
// * of browsers and screen readers. Oh, and there's also a clearfix! 
// * 
// * @ABOUT
// * All the classes comes from HTML5 Boilerplate (-> html5boilerplate.com). Here though,
// * they're changed so that they can be easily used in The LESS Way.
// * They're called "non-semantic" in H5B, but here they become semantic if used well.
// * (Actually they can't be used in any way than good in 3L.)
// * (Actually II they can be semantic in H5B as well, but it's a bit pain to do that.)
// * 
// * @USAGE
// * Just put them in your classes, id's or elements.
// * 
// * @EXAMPLE
// * .class-selector {
// *     .ir();
// *     .clearfix();
// * }
// **/

// Image replacement.
.ir () {border: 0; overflow: hidden; background-color: transparent; *text-indent: -9999px; &:before {content: ""; display: block; width: 0; height: 100%;}}

// Hide from both screenreaders and browsers: h5bp.com/u
.hidden () {display: none !important; visibility: hidden;}

// Hide only visually, but have it available for screenreaders: h5bp.com/v
.visuallyhidden () {border: 0;clip: rect(0 0 0 0);height: 1px;margin: -1px;overflow: hidden;padding: 0;position: absolute;width: 1px;}
	
// Extends the .visuallyhidden class to allow the element to be focusable when navigated to via the keyboard: h5bp.com/p
// CAUTION! The .visuallyhidden class is included! If you want your object to be both
// .visuallyhidden and .focusabe use only .focusable class.
.focusable () {.visuallyhidden;&:active,&:focus{clip: auto; height: auto; margin: 0; overflow: visible; position: static; width: auto;}}

// Hide visually and from screenreaders, but maintain layout.
.invisible () {visibility: hidden;}

// Contain floats: h5bp.com/q
.clearfix () {*zoom:1;&:before,&:after{content:" ";display:table;}&:after{clear:both;}}

///**
// * Some other helper classes.
// **/

.incomplete() {outline: 3px dotted green}
.fixme() {outline: 3px dotted yellow}
.todo() {outline: 3px dotted blue}
.xxx() {outline: 3px dotted red}

///**
// * Font-face
// * 
// * Easily include fonts on you website.
// *
// * @USAGE
// * Put the .font-face() mixin in your CSS. Provide at least two arguments:
// * -- font's name (string),
// * -- font's filename (without filetype),
// * -- font's path / URL (string) [don't forget to put the fonts in all formats in the folder].
// *
// * Optionally you can provide the font's weight and style. By default they're normal.
// *
// * Font's formats you need to use: EOT, WOFF, TTF and SVG.
// **/

.font-face(@name, @filename, @path, @weight:normal, @style:normal) {
	@font-face {
		font-family: @name;
		src: url('@{path}@{filename}.eot');
		src: local('☺'), url('@{path}@{filename}.woff') format('woff'), url('@{path}@{filename}.ttf') format('truetype'), url('@{path}@{filename}.svg') format('svg');
		font-weight: @weight;
		font-style: @style;
	}
}

///**
// * Box-sizing
// * 
// * Change the basic box-model to the one you want.
// * 
// * @ABOUT
// * Basic box-model defines the width and height of an object only as a size of
// * object's content area. In order to know exactly how big is the object you need
// * to add its padding and borders. This may result in many unwanted behaviours.
// * That's why you can define how you want the browser to calculate the width/height:
// * with or without padding, with or without borders.
// * 
// * @USAGE
// * To use this feature just put a value you want:
// * -- content-box ("content" works too) - it's default value from basic model.
// * -- padding-box (also "padding") - width and height declarations will include paddings.
// * -- border-box (and "border") - border and padding included.
// * 
// * 1. .box-sizing(content-box), .box-sizing(content) and .content-box-sizing for box-sizing: content-box.
// * 2. .box-sizing(padding-box), .box-sizing(padding) and .padding-box-sizing for box-sizing: padding-box.
// * 3. .box-sizing(border-box), .box-sizing(padding) and .border.box-sizing for box-sizing: border-box.
// *
// * @BROWSERS SUPPORS
// * IE8+ and every other.
// * Only Firefox supports padding-box.
// * 
// * @LINKS
// * * http://paulirish.com/2012/box-sizing-border-box-ftw/
// * * https://developer.mozilla.org/en-US/docs/CSS/box-sizing
// * * http://caniuse.com/#feat=css3-boxsizing
// **/

.content-box-sizing(){-moz-box-sizing:content-box;-webkit-box-sizing:content-box;box-sizing:content-box;}
.padding-box-sizing(){-moz-box-sizing:padding-box;}
.border-box-sizing(){-moz-box-sizing:border-box;-webkit-box-sizing:border-box;box-sizing:border-box;}
.box-sizing (padding){.padding-box-sizing;}
.box-sizing (padding-box){.padding-box-sizing;}
.box-sizing (border){.border-box-sizing;}
.box-sizing (border-box){.border-box-sizing;}
.box-sizing (content){.content-box-sizing;}
.box-sizing (content-box){.content-box-sizing;}

///**
// * Background-clip
// * 
// * Clips the object's background to the desired box according to the box-model.
// * 
// * @USAGE
// * To use this class simply put into an object a .background-clip(box) class,
// * where box means the desired box (see below). You can also use a shorter class
// * .bg-clip or class that corresponds to the clipping you want to get (.content-clip,
// * .padding-clip, .border-clip).
// * 
// * Arguments that comes into (box) are:
// * -- content-box or content for content clip,
// * -- padding-box or padding for padding clip,
// * -- border-box or border for border clip.
// * *** You can use values either with or without "-box" suffix.
// *
// * @BROWSERS SUPPORT
// * IE9+ and every other.
// *
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/background-clip
// * * http://caniuse.com/#feat=background-img-opts
// **/

.content-box(){-moz-background-clip:content;background-clip:content-box;}
.padding-box(){-moz-background-clip:padding;background-clip:padding-box;}
.border-box(){-moz-background-clip:border;background-clip:border-box;}
.background-clip (padding){.padding-box;}
.background-clip (padding-box){.padding-box;}
.background-clip (border){.border-box;}
.background-clip (border-box){.border-box;}
.background-clip (content){.content-box;}
.background-clip (content-box){.content-box;}
.bg-clip(@arguments){.background-clip(@arguments);}

///**
// * Box-shadow
// *
// * Create a shadow behind or inside the element.
// * 
// * @USAGE
// * In .box-shadow() brackets put arguments for every single shadow. Separate
// * each shadow's arguments with comma.
// * 
// * Box-shadow property takes following arguments:
// *
// * 1. inset [optional].
// * 2. x-offset [required].
// * 3. y-offset [required].
// * 4. blur [optional]. 
// * 5. spread [optional].
// * 6. color [optional/required].
// *
// * @EXAMPLES
// * Two box-shadows: one is 1px offset, black, second one is green with 30% opacity, inset,
// * with 5px offsets, 3px of blur and 1px spread.
// *
// * .box-shadow(1px 1px black, inset 5px 5px 3px 1px fade(green,30%))
// *
// * @BROWSERS SUPPORT
// * IE9, Fx3.5 and every other.
// *
// * @LINKS
// * * hhtp://developer.mozilla.org/en/CSS/box-shadow
// * * http://caniuse.com/#feat=css-boxshadow
// * * http://www.toekneestuck.com/blog/2012/05/15/less-css-arguments-variable/
// **/

.box-shadow(@shadow1, @shadow2:X, ...){
	@shadows: ~`"@{arguments}".replace(/[\[\]]|\,\sX/g, '')`;
    -webkit-box-shadow: @shadows;
    -moz-box-shadow: @shadows;
    box-shadow: @shadows;
}

///**
// * Border-radius
// *
// * Round the element's corners.
// * 
// * @USAGE
// * Border-radius property takes one to four arguments with px, em and %
// * values and round the element's corners accordingly. You can make
// * ellipticaly rounded corners putting two sets of values separated with
// * comma using .elliptical-border-radius class. In order to round a single
// * corner use .round-corner class. This class takes two sets of arguments.
// * First one is a declaration of corner you want to round (top-left, top-right,
// * bottom-right, bottom-left), second one (separated from the first with comma),
// * is a set of values in px, em or % that round chosen corner. Second
// * declaration can have one (for circle rounding) or two values (for
// * elliptical rounding).
// *
// * @EXAMPLES
// * 1. Round every corner of the element with 10px radius.
// *		.border-radius(10px)
// * 2. Round top-left and bottom-right corners by 10px, top-right
// *		and bottom-left corners by 20px.
// *		.border-radius(10px 20px)
// * 3	Exemplary use of the .elliptical-border-radius class.
// * 		.elliptical-border-radius(10px 20px 30px, 40px 50px 60px 70px)
// * 4. Round top-right corner by 10px and 20px (elliptical).
// * 		.round-corner(top-right, 10px 20px)
// *		.border-top-right-radius(10px 20px) // Alternate method.
// * 5. Round similar corners.
// * 		.border-top-radius(20px 10px); // top-left + top-right (elliptical)
// *		.border-left-radius(5px);		 // top-left + bottom-left
// *
// * @BROWSERS SUPPORT
// * IE9, Fx3.5 and every other
// * 
// * @LINKS
// * * http://developer.mozilla.org/en/CSS/border-radius
// * * http://caniuse.com/#feat=border-radius
// **/

// If your rounded corners looks bad with borders, add this class to your rounded element.
// * http://tumble.sneak.co.nz/post/928998513/fixing-the-background-bleed
.border-radius-fix(){.background-clip(padding-box);}

.border-radius (@radius:5px, ...) {
	-webkit-border-radius: @arguments;
	-moz-border-radius: @arguments;
	border-radius: @arguments;
}
// Alternate name for .border-radius.
.round-corners (@radius:5px, ...) {.border-radius(@arguments);}

.elliptical-border-radius (@radius1, @radius2) {
	-webkit-border-radius: @radius1 ~"/" @radius2;
	-moz-border-radius: @radius1 ~"/" @radius2;
	border-radius: @radius1 ~"/" @radius2;
}
.round-corner (top-left, @radius...){
	-webkit-border-top-left-radius: @radius;
	-moz-border-top-left-radius: @radius;
	border-top-left-radius: @radius;
}
.round-corner (top-right, @radius...){
	-webkit-border-top-right-radius: @radius;
	-moz-border-top-right-radius: @radius;
	border-top-right-radius: @radius;
}
.round-corner (bottom-right, @radius...) {
	-webkit-border-bottom-right-radius: @radius;
	-moz-border-bottom-right-radius: @radius;
	border-bottom-right-radius: @radius;
}
.round-corner (bottom-left, @radius...) {
	-webkit-border-bottom-left-radius: @radius;
	-moz-border-bottom-left-radius: @radius;
	border-bottom-left-radius: @radius;
}
// Another methods to use corner radius.
.border-top-left-radius (...) {.round-corner(top-left,@arguments);}
.border-top-right-radius (...) {.round-corner(top-right,@arguments);}
.border-bottom-right-radius (...) {.round-corner(bottom-right,@arguments);}
.border-bottom-left-radius (...) {.round-corner(bottom-left,@arguments);}

// Round similar corners.
.border-top-radius (...) {.round-corner(top-left,@arguments);.round-corner(top-right,@arguments);}
.border-bottom-radius (...) {.round-corner(bottom-left,@arguments);.round-corner(bottom-right,@arguments);}
.border-left-radius (...) {.round-corner(top-left,@arguments);.round-corner(bottom-left,@arguments);}
.border-right-radius (...) {.round-corner(top-right,@arguments);.round-corner(bottom-right,@arguments);}

// Another classes for the same purpose as above.
.round-top-corners (...) {.border-top-radius(@arguments);}
.round-bottom-corners (...) {.border-bottom-radius(@arguments);}
.round-left-corners (...) {.border-left-radius(@arguments);}
.round-right-corners (...) {.border-right-radius(@arguments);}

///**
// * Opacity
// *
// * Make an object transparent.
// *
// * @ABOUT
// * Opacity takes values between 0.0 (invisible) to 1.0 (default - full
// * visibility) but 3L lets you also use percentages and values from >1 to 100.
// *
// * @USAGE
// * If you want an element with transparency 1, .5 or 0, use these classes:
// * .not-transparent., .half-transparent, .transparent.
// *
// * @BROWSERS SUPPORT
// * full (IE6+)
// *
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/opacity
// * * http://caniuse.com/#feat=css-opacity
// * 
// * @ASIDE
// * Do we need the ability to set opacity in numbers from 1 to 100
// * and in percentages? I think we do, because:
// * 1. You need to use integer numbers in filter property for IE,
// *    so there is a possibility that someone will type this kind
// *    of value, instead of [0,1].
// * 2. LESS has a fade() function that uses percentages to makes
// *    colours (semi)transparent, so using percentages here will
// *    result in more consistent code (same unit in similar situation).
// * 3. Percentages are more intuitive when it comes to transparency and
// *    opacity since graphic editors like Photoshop and GIMP use them.
// *
// **/

.transparent() {.opacity(0);}
.non-transparent() {.opacity(1);}
.half-transparent () {.opacity(.5);}
.opacity () {.non-transparent();}
.opacity (@value) when (isnumber(@value)) and (@value =< 1){
	opacity: @value;
	filter: ~"alpha(opacity="@value*100~")";
	}
.opacity (@value) when (isnumber(@value)) and (@value > 1) and not (ispercentage(@value)){
	// INFO: for the explanation of rule "and not (ispercentage(@value))" refer to the class below.
	opacity: @value/100;
	filter: ~"alpha(opacity="@value~")";
	}
.opacity (@value) when (ispercentage(@value)) {
	// Change the @value from percentage to integer (XX% => XX)
	@integerValue: `parseInt('@{value}')`;
	// Actually the @value should be a number now... but it isn't.
	// LESS think of it as a string so we can't do math here.
	// We'll use a feature of LESS that automatically makes
	// a second value's unit in a sum to be the same as the first one.
	// In our case we need a number so we can divide it by 100.
	opacity: (0 + @integerValue) /100;
	// We don't need a math in filter so no trick.
	filter: ~"alpha(opacity="@integerValue~")";
	
	// BUG: WinLESS compile this with doubled properties unless the rule
	// "and not (ispercentage(@value))" is added in the class above.
	// In Firebug (through the LESS native compiler) those properties
	// aren't doubled, though.
	}

///**
// * Gradient
// *
// * Create a beautiful gradient without images.
// *
// * @USAGE
// * Pick a type of a gradient you want to have and put colours as arguments.
// * You can put either two or three colours. In the first case
// * you type only the gradient's colours. Three arguments gradient mixins
// * will use the third colour as background-color.
// *
// * The default class - .gradient - is the same as .vertical-gradient.
// *
// * You can choose from four types of gradients:
// * 1. Vertical - colour will change from top to bottom.
// * 2. Horizontal - change is from left to right.
// * 3. Diagonal - from top-left to bottom-right.
// * 4. Radial - from the centre of an element to its borders.
// *
// * @BROWSERS SUPPORT
// * IE10 and every other except Opera Mini.
// * Some gradients can be achieved in older IE by using non-standard properties.
// * 
// * @LINKS
// * * http://www.colorzilla.com/gradient-editor/ -- gradient generator with broad browsers support and some advanced options
// * * http://lea.verou.me/css3patterns/ -- gradient is a powerful tool - you can generate shapes as a background!
// * * https://developer.mozilla.org/en-US/docs/CSS/gradient
// * * http://caniuse.com/#feat=css-gradients
// *
// **/

.gradient (@color1, @color2){
	background-image: -moz-linear-gradient(top, @color1 0%, @color2 100%);
	background-image: -webkit-linear-gradient(top, @color1 0%,@color2 100%);
	background-image: linear-gradient(to bottom, @color1 0%,@color2 100%);
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='@{color1}', endColorstr='@{color2}',GradientType=0 );
}
.gradient (@color1, @color2, @color3){
	background-color: @color3;
	background-image: -moz-linear-gradient(top, @color1 0%, @color2 100%);
	background-image: -webkit-linear-gradient(top, @color1 0%,@color2 100%);
	background-image: linear-gradient(to bottom, @color1 0%,@color2 100%);
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='@{color1}', endColorstr='@{color2}',GradientType=0 );
}
// .vertical-gradient = .gradient
.vertical-gradient (@color1,@color2) {.gradient(@color1,@color2)}
.vertical-gradient (@color1,@color2,@color3) {.gradient(@color1,@color2,@color3)}
.horizontal-gradient (@color1, @color2) {
	background-image: -moz-linear-gradient(left, @color1 0%, @color2 100%);
	background-image: -webkit-linear-gradient(left, @color1 0%,@color2 100%);
	background-image: linear-gradient(to right, @color1 0%,@color2 100%);
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='@{color1}', endColorstr='@{color2}',GradientType=1 );
}
.horizontal-gradient (@color1, @color2, @color3) {
	background-color: @color3;
	background-image: -moz-linear-gradient(left, @color1 0%, @color2 100%);
	background-image: -webkit-linear-gradient(left, @color1 0%,@color2 100%);
	background-image: linear-gradient(to right, @color1 0%,@color2 100%);
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='@{color1}', endColorstr='@{color2}',GradientType=1 );
}
.diagonal-gradient (@color1, @color2) {
	background-image: -moz-linear-gradient(-45deg, @color1 0%, @color2 100%);
	background-image: -webkit-linear-gradient(-45deg, @color1 0%,@color2 100%);
	background-image: linear-gradient(135deg, @color1 0%,@color2 100%);
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='@{color1}', endColorstr='@{color2}',GradientType=1 );
	}
.diagonal-gradient (@color1, @color2, @color3) {
	background-color: @color3;
	background-image: -moz-linear-gradient(-45deg, @color1 0%, @color2 100%);
	background-image: -webkit-linear-gradient(-45deg, @color1 0%,@color2 100%);
	background-image: linear-gradient(135deg, @color1 0%,@color2 100%);
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='@{color1}', endColorstr='@{color2}',GradientType=1 );
	}
.radial-gradient (@color1, @color2) {
	background-image: -moz-radial-gradient(center, @color1 0%, @color2 100%);
	background-image: -webkit-radial-gradient(center, @color1 0%,@color2 100%);
	background-image: radial-gradient(at center, @color1 0%,@color2 100%);
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='@{color1}', endColorstr='@{color2}',GradientType=1 );
	}
.radial-gradient (@color1, @color2, @color3) {
	background-color: @color3;
	background-image: -moz-radial-gradient(center, @color1 0%, @color2 100%);
	background-image: -webkit-radial-gradient(center, @color1 0%,@color2 100%);
	background-image: radial-gradient(at center, @color1 0%,@color2 100%);
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='@{color1}', endColorstr='@{color2}',GradientType=1 );
	}

// !! WARNING !!
// MULTIPLE ARGUMENTS GRADIENT MIXINS IS IN DEVELOPEMENT AND PROBABLY WON'T WORK.
//
// * TODO: test if the beta-gradient is reverse-compatible with the old gradient.
// * FIXME: you don't need to provide the @direction value since default is "to bottom". How to make it works?
// * FIXME: simple radial-gradient should work, but what if you provide some more advanced values?
// * FIXME: does it work with "to top left" and similar or with angles?

// .gradient(@gradientType, @direction, @rest...) {
	// @valuesProcessed: ~`"@{rest}".replace(/[\[\]]/g, '')`;
	// @directionProcessedOld: ~`"@{direction}".replace(/[\[\]]|\,/g,'').replace("to ","").replace("top","bottom").replace("bottom","top").replace("right","left").replace("left","right").replace("at ","").replace("cover", "farthest-corner")`;
	// @directionProcessedNew: ~`"@{direction}".replace(/[\[\]]|\,/g,'')`; 
// 	
	// @webkit1: `"-webkit-" + "@{gradientType}" + "-gradient(" + "@{directionProcessedOld}," + "@{valuesProcessed}" + ")"`;
	// @webkit2: ~`'@{webkit1}'.replace(/\"/g, '')`;
	// background-image: @webkit2;
// 	
	// @moz1: `"-moz-" + "@{gradientType}" + "-gradient(" + "@{directionProcessedOld}," + "@{valuesProcessed}" + ")"`;
	// @moz2: ~`'@{moz1}'.replace(/\"/g, '')`;
	// background-image: @moz2;
// 	
	// @w3c1: `"@{gradientType}" + "-gradient(" + "@{directionProcessedNew}," + "@{valuesProcessed}" + ")"`;
	// @w3c2: ~`'@{w3c1}'.replace(/\"/g, '')`;
	// background-image: @w3c2;
// }

///**
// * Background-size
// * 
// * Scale (or not) your background image.
// * 
// * @ABOUT
// * This property takes following values:
// * -- auto [default] - it does nothing when used alone; when used with a value
// *    it makes sure that image will keep its aspect ratio while being stretched
// *    to the required size. (See: examples 3. and 4.)
// * -- contain - scale image to the first border it meets;
// *    it may leave some area uncovered but keeps image's aspect ratio,
// * -- cover - scale image to the second border it meets;
// *    cover all area but part of an image may not be shown. The image keeps its aspect ratio.
// * -- px, em, % - scale image according to declared value;
// *    you can declare one value (x-size) or two values for each size.
// *    Using (100%, 100%) stretch the image to cover full area but may not
// *    keep its aspect ratio. When you use percentages keep in mind that XX%
// *    means XX% of the element size, not XX% of the background image. 
// * 
// * Consider adding background-repeat property to avoid unwanted repetition of background.
// *
// * @EXAMPLES
// * 1. .background-size(contain);
// * 2. .background-size(cover); 
// * 3. .background-size(70%); 		 // = (70% auto). Image is scaled to take 70% of width
// *									 		 // of the element and it keeps its own aspect ratio.
// * 4. .background-size(auto, 70%) // Image is scaled to take 70% of element's
// * 										 // height and keeps aspect ratio.
// * 5. background-size(70px 7em);  // Size of the background image is now 70px (width) x 7em (height).
// * 6. background-size(70px,7em);  // Same as above. Comma is fine too.
// *
// * @BROWSERS SUPPORT
// * IE9+, Fx3.6+ and every other.
// *
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/background-size
// * * http://caniuse.com/#search=background-size
// **/
 
.background-size (...) {
	-moz-background-size: @arguments;
	background-size: @arguments;
	}
// A shorthand class.
.bg-size (...) {.background-size(@arguments)}

///**
// * Columns layout
// * 
// * Divide a block of text into columns as seen in newspapers.
// * 
// * @USAGE
// * Use .columns() class providing arguments in the brackets.
// * None of those arguments are required, but unless you provide one, columns
// * layout won't work. Available arguments:
// * -- [integer] - declare a column-count. Unless declared, their width will 
// *    equal to division of the block's width minus column-gaps and declared integer.
// *    Default value is "auto" which means that width of columns will be determined
// *    by column-width value.
// * -- [width] - declare every column width in px, em and % (of containing block).
// *    Default value is "auto" - column width will be equal to division of the 
// *    block's width and declared column-count
// * 
// * Usage of supporting classes:
// * You can declare the gaps between columns by .column-gap() class that takes
// * width type argument. Default value is "normal" and equals to 1em.
// * 
// * Declare a vertical rule between columns using .column-rule(). It's the same
// * type of declaration as in borders, that is width, style and colour. By default
// * there is no rule.
// * 
// * @EXAMPLES
// * 1. .column(2, 20px) // Two columns with 20px width.
// * 2. .column(5) // Five columns layout.
// * 3. element {
// *        .column(100px);					// A 100px width columns
// * 		.column-gap(10px);				// with 10px gap between
// * 		.column-rule(1px solid black);	// and 1px thick, solid, black vertical rule.
// *	}
// *
// * @BROWSERS SUPPORT
// * IE10+ and every other (except for Opera Mini)
// * 
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/columns
// * * http://caniuse.com/#feat=multicolumn
// **/

.columns (...) {
	-webkit-columns: @arguments;
	-moz-columns: @arguments;
	columns: @arguments;
	}
.column-gap (@gap) {
	-webkit-column-gap: @gap;
	-moz-column-gap: @gap;
	column-gap: @gap;
	}
.columns-gap (@gap) {.column-gap(@gap);}
.column-rule (...) {
	-webkit-column-rule: @arguments;
	-moz-column-rule: @arguments;
	column-rule: @arguments;
	}
.columns-rule (...) {.column-rule(@arguments);}
.column-fill (@fill) {
	-webkit-column-fill: @fill;
	-moz-column-fill: @fill;
	column-fill: @fill;
}
.columns-fill (@fill) {.column-fill(@fill);}

///**
// * Transforms
// * 
// * 2D and 3D transformation of an object.
// *
// * @ABOUT
// * Transforms don't affect an object's place in a document and its environment
// * so there's no risk of crashing a layout.
// * 
// * Transformations characteristics:
// * -- Rotate takes one argument and rotate object clockwise by specified angle (in deg).
// * -- Scale takes one or two arguments (x,y). If y is not specified it is assumed that x = y.
// * 	Arguments in interval (0,1) shrink the object. Arguments >1 makes it bigger.
// * -- Skew takes one or two arguments (x,y). If y is not specified it is assumed that y = 0 (no y-skew).
// * 	Arguments must be in deg.
// * -- Translate takes one or two arguments (w,y). If y is not specified it is assumed y = 0 (no y-translate).
// * 	Arguments are in px or em. Translate moves the object by specified value.
// * 
// * Sometimes you might want to control the point that is the relative base of transformations.
// * For that reason use .transform-origin class. That class takes one or two values.
// * First value defines horizontal position of that point, second refers to vertical position.
// * In case you provide only the first value, the second is set to 50%. Default value is (50% 50%).
// * You can use three types of values:
// * -- pixels that place origin point in position according to the top-left corner of an element
// * 	to its bottom-right corner. Negative values allowed - in that case position goes to the left
// *    and top from the top-left corner,
// * -- percentages that are relative to object's width and height. As in px, the original position
// *    is top-left corner (0% 0%), and goes to bottom-right (100% 100%). Negative values does the
// *    same as in px.
// * -- keywords: left / center / right for x-axis and top / center / bottom for y-axis.
// *
// * For 3D transformations you can set a perspective using .perspective() class and putting
// * value in brackets .
// * 
// * @USAGE
// * You can use classes dedicated to specifics transformation, but if you plan
// * to use multiple transformation on an object, use general classes .transform
// * or .transform3d instead. Otherwise the latter transform will override the former.
// * If you want to use some of the 3D transformations, use .transform3d class. It
// * has a 3D specific property transform-style included. You can put both 3D and 2D
// * transformations in .transform3D.
// * 
// * @EXAMPLES
// * 1. .scale(2,.5) // Stretch an object two times and shrink in height by half.
// * 2. .rotate(180deg) // Rotate an object by 180deg.
// * 3. .transform(scale(2,.5) rotate(180deg)) // Does the combined transformation from examples above.
// * 4. .transform-origin(20% top) // Place the transformation origin at the top, 20% of the object's
// * 							   // width to the right from the top-left corner. 
// * 	
// * @BROWSERS SUPPORT
// * IE9 (2D), IE10+ (3D), Fx3.5 (2D) and every other (except for Opera Mini)
// * 
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/transform
// * * http://caniuse.com/#feat=transforms2d
// * * http://caniuse.com/#feat=transforms3d
// **/

.transform-origin (...) {
	-webkit-transform-origin: @arguments;
	-moz-transform-origin: @arguments;
	-ms-transform-origin: @arguments;
	transform-origin: @arguments;
}
.transform-style(@style) {
	-webkit-transform-style: preserve-3d;
	-o-transform-style: preserve-3d;
	transform-style: preserve-3d;
}
.perspective (@perspective) {
	-webkit-perspective: @perspective;
	perspective: @perspective;
}
.perspective-origin (...) {
	-webkit-perspective-origin: @arguments;
	perspective-origin: @arguments;
}
.backface-visibility(@visibility){
    -webkit-backface-visibility: @visibility;
    backface-visibility: @visibility;
}
.transform (...) {
	-webkit-transform: @arguments;
	-moz-transform: @arguments;
	-ms-transform: @arguments;
	transform: @arguments;
	}
.transform3d (...) {
	-webkit-transform: @arguments;
	transform: @arguments;
}
.transform-perspective (@perspective) {
	-webkit-transform: perspective(@perspective);
	transform: perspective(@perspective);
}
.rotate (@rotate) {
	-webkit-transform: rotate(@rotate);
	-moz-transform: rotate(@rotate);
	-ms-transform: rotate(@rotate);
	transform: rotate(@rotate);
	}
.rotate3d (@deg1, @deg2:0, @deg3:0){
	-webkit-transform: rotateX(@deg1) rotateY(@deg2) rotateZ(@deg3);
	transform: rotateX(@deg1) rotateY(@deg2) rotateZ(@deg3);
	}
.scale (@scale) {
	-webkit-transform: scale(@scale);
	-moz-transform: scale(@scale);
	-ms-transform: scale(@scale);
	transform: scale(@scale);
	}
.scale (@scale1,@scale2) {
	-webkit-transform: scale(@scale1,@scale2);
	-moz-transform: scale(@scale1,@scale2);
	-ms-transform: scale(@scale1,@scale2);
	transform: scale(@scale1,@scale2);
	}
.scaleX (@scale) {
	-webkit-transform: scaleX(@scale);
	-moz-transform: scaleX(@scale);
	-ms-transform: scaleX(@scale);
	transform: scaleX(@scale);
	}
.scaleY (@scale) {
	-webkit-transform: scaleY(@scale);
	-moz-transform: scaleY(@scale);
	-ms-transform: scaleY(@scale);
	transform: scaleY(@scale);
	}
.skew (@skew) {
	.skewX(@skew);
	}
.skew (@skew1, @skew2) {
	-webkit-transform: skewX(@skew1) skewY(@skew2);
	-moz-transform: skewX(@skew1) skewY(@skew2);
	-ms-transform: skewX(@skew1) skewY(@skew2);
	transform: skewX(@skew1) skewY(@skew2);
	}
.skewX (@skew) {
	-webkit-transform: skewX(@skew);
	-moz-transform: skewX(@skew);
	-ms-transform: skewX(@skew);
	transform: skewX(@skew);
	}
.skewY (@skew) {
	-webkit-transform: skewY(@skew);
	-moz-transform: skewY(@skew);
	-ms-transform: skewY(@skew);
	transform: skewY(@skew);
	}
.translate (@translate) {
	-webkit-transform: translate(@translate);
	-moz-transform: translate(@translate);
	-ms-transform: translate(@translate);
	transform: translate(@translate);
	}
.translate (@translate1, @translate2) {
	-webkit-transform: translate(@translate1, @translate2);
	-moz-transform: translate(@translate1, @translate2);
	-ms-transform: translate(@translate1, @translate2);
	transform: translate(@translate1, @translate2);
	}
.translateX (@translate) {
	-webkit-transform: translateX(@translate);
	-moz-transform: translateX(@translate);
	-ms-transform: translateX(@translate);
	transform: translateX(@translate);
	}
.translateY (@translate) {
	-webkit-transform: translateY(@translate);
	-moz-transform: translateY(@translate);
	-ms-transform: translateY(@translate);
	transform: translateY(@translate);
	}
	
///**
// * Transition
// * 
// * Animate a change between different object states.
// * 
// * @ABOUT
// * You can use this class to animate change of up to 5 different properties.
// * If you need more that that (wow!) it's easy to add more of these classes.
// * 
// * Transition takes 4 values:
// * -- transition-property - choose a property you want to animate (margin, colour etc.).
// * 	Default value is "all" which will animate every change that might happen. It's a good
// * 	idea to explicitly write this value if you want to animate everything, though. We can't
// * 	be sure if in the future default state won't change to "none", which will break the whole transition.
// * -- transition-duration - specifies how long transition animation will have to take
// *    until animation is finished. Put values in s (seconds) or ms (milliseconds).
// * 	Required value, since default state is 0s (no animation occurs).
// * -- transition-timing-function - this value describe an acceleration function. It can
// *    really affect overall experience, so it's good idea to pay attention to that.
// * 	Timing function takes 4 number values of keywords: linear, ease, ease-in,
// * 	ease-in-out, ease-out. For further explanation refer to ->developer.mozilla.org/en/CSS/timing-function
// * 	Default value is "ease".
// * -- transition-delay - in seconds or millisecond describes how long transition will wait
// * 	until it occurs after a triggering requirement was met. Default value is 0 - instant animation.
// * 
// * In case you want to put a transformation property into transition effect, use
// * the .transition-transform class. This class is prepared to automatically add
// * many browsers prefixes required in that case. It is also a bit future friendly,
// * but all at all using transitions on transform is very risky.
// * -> http://radiatingstar.com/transition-with-transform-cant-be-future-proof
// * 
// * @USAGE
// * For .transition class, put in brackets at least a time value. If you want to animate more than one
// * property or animate everything with different functions, put up to 5 declarations separated with
// * commas. If one of the property you want to animate is "transform" _and_ you want to animate every
// * other properties (or at least not with the same functions), use .transition-transform. As a first
// * value put a time duration for transform property. Do not write this property though. It's auto-added
// * for the first set of values. Add other properties after commas. Refer to the example 3.
// * 
// * @EXAMPLES
// * 1. .transition (all 1s); // Animate all properties for 1 second.
// * 2. .transition (background-color 5s, margin 1s linear 5s) // Animate change of background-color for 5s
// * 														   // and after 5s animate margin change for 1s.
// * 3. .transition-transform (2s linear, padding 3s) // Animate change of transform property (no "transform" declared!)
// * 												  // and padding property.
// * 
// * @BROWSERS SUPPORT
// * IE10+ and every other (except for Opera Mini)
// * 
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/transition
// * * http://caniuse.com/#feat=css-transitions
// **/

.transition(@transition1, @transition2:X, ...){
	@transitions: ~`"@{arguments}".replace(/[\[\]]|\,\sX/g, '')`;
    -webkit-transition: @transitions;
    transition: @transitions;
}
.transition-transform (@transformArguments1, @transformArguments2:X, ...) {
	@transformArguments: ~`"@{arguments}".replace(/[\[\]]|\,\sX/g, '')`;
	-webkit-transition: -webkit-transform @transformArguments;
	transition: transform @transformArguments;
}
.transition-property(@property1, @property2:X, ...) {
	@properties: ~`"@{arguments}".replace(/[\[\]]|\,\sX/g, '')`;
	-webkit-transition-property: @properties;
	transition-property: @@properties;
}
.transition-duration(@property1, @property2:X, ...) {
	@properties: ~`"@{arguments}".replace(/[\[\]]|\,\sX/g, '')`;
	-webkit-transition-duration: @properties;
	transition-duration: @properties;
}
.transition-timing-function(@property1, @property2:X, ...) {
	@properties: ~`"@{arguments}".replace(/[\[\]]|\,\sX/g, '')`;
	-webkit-transition-timing-function: @properties;
	transition-timing-function: @properties;
}
.transition-delay(@property1, @property2:X, ...) {
	@properties: ~`"@{arguments}".replace(/[\[\]]|\,\sX/g, '')`;
	-webkit-transition-delay: @properties;
	transition-delay: @properties;
}

///**
// * Animations
// * 
// * Create an awesome animation!
// * 
// * @ABOUT
// * This class takes from two to five properties:
// * -- animation-name [required] - declare your @keyframes animation name. See below
// * 	for an easy way to make @keyframes!
// * -- animation-duration [required] - declare how long will it take for an animation
// * 	to reach end. Value in seconds (s) or milliseconds (ms).
// * -- timing-function - it's the same property as in transition. Refer to
// * 	that topic for further explanation.
// * -- iteration-count - how many times an animation will repeat. It takes integer
// * 	or "infinite" keyword (for infinite repetition). Default value is 1.
// * -- direction - indicates whether the animation should play in reverse on
// * 	alternate cycles. Refer to ->developer.mozilla.org/en/CSS/animation-direction
// * 	for better explanation. To declare a direction use normal [default value],
// * 	alternate, reverse or alternate-reverse keyword.
// * 
// * @USAGE
// * Creating @keyframes
// * Write in your .less file:
// * *** @import 'animationX.less';
// * where X stands for a number between 1 and 5. Then create a class
// * *** .animationX () {}
// * and in {} write declarations you normally write in @keyframes.
// * Then just put an .animation(animationX [other animation properties]) class in your element.
// * That's all!
// *
// * @EXAMPLES
// * .toBeAnimated {
// * 	// Animation with every possible value declared. //
// *	.animation(animation1 5s linear 3s infinite alternate);
// * }
// * // Importing prefixed @keyframes for animation1.
// * @import '3L/assets/animations/animation1';
// *
// * // Declaring @keyframes for animation. Only once! 
// * .animation1() {
// *	from {margin-top: 3px;}
// *	to {margin-top: 333px;}
// * }
// *
// * @BROWSERS SUPPORT
// * IE10+ and every other (except for Opera Mini)
// *
// * @LINKS
// * * http://developer.mozilla.org/en/CSS/animation
// * * http://caniuse.com/#feat=css-animation
// **/

.animation (...) {
	-webkit-animation: @arguments;
	animation: @arguments;
	}
.animate (...) {.animation(@arguments);}
.anime (...) {.animation(@arguments);}
.animation-name(...) {
	-webkit-animation-name: @arguments;
	animation-name: @arguments;
}
.animation-duration(...) {
	-webkit-animation-duration: @arguments;
	animation-duration: @arguments;
}
.animation-timing-function(...) {
	-webkit-animation-timing-function: @arguments;
	animation-timing-function: @arguments;
}
.animation-delay(...) {
	-webkit-animation-delay: @arguments;
	animation-delay: @arguments;
}
.animation-iteration-count(...) {
	-webkit-animation-iteration-count: @arguments;
	animation-iteration-count: @arguments;
}
.animation-direction(...) {
	-webkit-animation-direction: @arguments;
	animation-direction: @arguments;
}
.animation-fill-mode(...) {
	-webkit-animation-fill-mode: @arguments;
	animation-fill-mode: @arguments;
}

///**
// * User-select
// *
// * Controls the selection model of an element.
// *
// * @ABOUT
// * This property takes following values:
// * -- none - none of the element's content can be selected.
// * -- text [default] - the element's contents follow a standard text content selection model.
// * -- toggle - the element's contents follow a standard toggling content model.
// * -- element - one element at a time may be selected.
// * -- elements - one or more elements at a time may be selected.
// * -- all - Only the entire contents as a whole can be selected.
// *
// * @BROWSERS SUPPORT
// * IE10+, Firefox, Chrome, Safari (not Opera)
// *
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/user-select
// * * http://caniuse.com/#search=user-select
// **/

.user-select (...) {
	-webkit-user-select: @arguments;
	-moz-user-select: @arguments;
	-ms-user-select: @arguments;
	user-select: @arguments;
}

///**
// * Filter
// * 
// * Advanced image manipulation in your CSS!
// * 
// * @NOTE
// * The filter property currently works only in Webkit browsers. You might want to 
// * just use the -webkit-filter property without the help of .filter() mixins to
// * avoid generating useless code.
// * 
// * @BROWSERS SUPPORT
// * Newest Webkit browsers.
// * 
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/filter
// * * http://www.html5rocks.com/en/tutorials/filters/understanding-css/
// **/

.filter(@filter1, @filter2:X, ...) {
	@filters: ~`"@{arguments}".replace(/[\[\]]|\,\sX/g, '')`;
	-webkit-filter: @filters;
	-moz-filter: @filters;
	-ms-filter: @filters;
	filter: @filters;
}
.grayscale(@amount) {
	-webkit-filter: grayscale(@arguments);
	-moz-filter: grayscale(@arguments);
	-ms-filter: grayscale(@arguments);
	filter: grayscale(@arguments);
}
.sepia(@amount) {
	-webkit-filter: sepia(@arguments);
	-moz-filter: sepia(@arguments);
	-ms-filter: sepia(@arguments);
	filter: sepia(@arguments);
}
.saturate(@amount) {
	-webkit-filter: saturate(@arguments);
	-moz-filter: saturate(@arguments);
	-ms-filter: saturate(@arguments);
	filter: saturate(@arguments);
}
.hue-rotate(@angle) {
	-webkit-filter: hue-rotate(@arguments);
	-moz-filter: hue-rotate(@arguments);
	-ms-filter: hue-rotate(@arguments);
	filter: hue-rotate(@arguments);
}
.invert(@amount) {
	-webkit-filter: invert(@arguments);
	-moz-filter: invert(@arguments);
	-ms-filter: invert(@arguments);
	filter: invert(@arguments);
}
// .opacity() already taken, hence the -filter suffix.
.opacity-filter(@amount) {
	-webkit-filter: opacity(@arguments);
	-moz-filter: opacity(@arguments);
	-ms-filter: opacity(@arguments);
	filter: opacity(@arguments);
}
.brightness(@amount) {
	-webkit-filter: brightness(@arguments);
	-moz-filter: brightness(@arguments);
	-ms-filter: brightness(@arguments);
	filter: brightness(@arguments);
}
.contrast(@amount) {
	-webkit-filter: contrast(@arguments);
	-moz-filter: contrast(@arguments);
	-ms-filter: contrast(@arguments);
	filter: contrast(@arguments);
}
.blur(@radius) {
	-webkit-filter: blur(@arguments);
	-moz-filter: blur(@arguments);
	-ms-filter: blur(@arguments);
	filter: blur(@arguments);
}
.drop-shadow(@shadow) {
	-webkit-filter: drop-shadow(@arguments);
	-moz-filter: drop-shadow(@arguments);
	-ms-filter: drop-shadow(@arguments);
	filter: drop-shadow(@arguments);
}
.url(@url) {
	-webkit-filter: (@arguments);
	-moz-filter: (@arguments);
	-ms-filter: (@arguments);
	filter: @arguments;
}

///**
// * Border image
// *
// * Use images as your borders.
// *
// * @ABOUT
// * Border image lets you use an image as borders for an element. It takes at least 3 arguments:
// * -- an URL of an imge that will be used as a border,
// * -- image-slice (number) that tells the browser how to cut the image,
// * -- image-repeat (keywords: repeat, round or stretch)
// *    and optionally:
// * -- image-width for defining an offset of border parts in an image provided in url,
// * -- offset that the border-image will extend beyond a box.
// *
// * A layout of a page isn't affected by border-image. It's calculated only based
// * on border-width property. That means you can use even a very wide image and
// * still the surrounding content won't move a pixel.
// *
// * @USAGE
// * In .border-image() provide at least required arguments with the first being the URL path (without "url()").
// * 
// * @BROWSERS SUPPORT
// * All except IE
// * 
// * @LINKS
// * * https://developer.mozilla.org/pl/docs/CSS/border-image
// * * http://caniuse.com/#feat=border-image
// * * http://dev.w3.org/csswg/css-backgrounds/#border-image
// **/

.border-image(@url, @rest...) {
	-webkit-border-image:url(@url) @rest;
	-moz-border-image:url(@url) @rest;
	-o-border-image:url(@url) @rest;
	border-image:url(@url) @rest;
}
.border-image-source(@url) {
	-webkit-border-image-source: url(@url);
	-moz-border-image-source: url(@url);
	-o-border-image-source: url(@url);
	border-image-source: url(@url);
}
.border-image-slice(@slice) {
	-webkit-border-image-slice: @slice;
	-moz-border-image-slice: @slice;
	-o-border-image-slice: @slice;
	border-image-slice: @slice;
}
.border-image-width(@width) {
	-webkit-border-image-width: @width;
	-moz-border-image-width: @width;
	-o-border-image-width: @width;
	border-image-width: @width;
}
.border-image-outset(@outset) {
	-webkit-border-image-outset: @outset;
	-moz-border-image-outset: @outset;
	-o-border-image-outset: @outset;
	border-image-outset: @outset;
}
.border-image-repeat(@repeat) {
	-webkit-border-image-repeat: @repeat;
	-moz-border-image-repeat: @repeat;
	-o-border-image-repeat: @repeat;
	border-image-repeat: @repeat;
}

///**
// * Flexible Box Model
// * 
// * @BROWSERS SUPPORT
// * Firefox 22+, Chrome, Opera
// * 
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/Using_CSS_flexible_boxes
// * * http://caniuse.com/#search=flex
// **/

.display-flex() {
	display: -webkit-flex;
	display: flex;
}
.display-inline-flex(){
	display: -webkit-inline-flex;
	display: inline-flex;
}
.flex-direction(@direction) {
	-webkit-flex-direction: @direction;
	flex-direction: @direction;
}
.justify-content(@alignment) {
	-webkit-justify-content: @alignment;
	justify-content: @alignment;
}
.align-content(@alignment) {
	-webkit-align-content: @alignment;
	align-content: @alignment;
}
.align-items(@alignment) {
	-webkit-align-items: @alignment;
	align-items: @alignment;
}
.align-self(@alignment) {
	-webkit-align-self: @alignment;
	align-self: @alignment;
}
.flex(@arguments){
	-webkit-flex: @arguments;
	flex: @arguments;
}
.flex-basis(@basis) {
	-webkit-flex-basis: @basis;
	flex-basis: @basis;
}
.flex-grow(@grow) {
	-webkit-flex-grow: @grow;
	flex-grow: @grow;
}
.flex-shrink(@shrink) {
	-webkit-flex-shrink: @shrink;
	flex-shrink: @shrink;
}
.flex-flow(@arguments) {
	-webkit-flex-flow: @arguments;
	flex-flow: @arguments;
}
.flex-direction(@direction) {
	-webkit-flex-direction: @direction;
	flex-direction: @direction;
}
.flex-wrap(@wrap) {
	-webkit-flex-wrap: @wrap;
	flex-wrap: @wrap;
}
.order(@order) {
	-webkit-order: @order;
	order: @order;
}

///**
// * Appearance
// *
// * @LINKS
// * * https://developer.mozilla.org/en-US/docs/CSS/-moz-appearance
// **/

.appearance (@appearance) {
    -webkit-appearance: @appearance;
    -moz-appearance: @appearance;
}

///**
// * Selection
// *
// * Style a visual representtion of selecting elements and objects.
// * 
// * @USAGE
// * You can use it on the root of your CSS to make the selection style global
// * or inside a specyfic element. Provide two arguments: font color and background-color.
// * 
// * @EXAMPLES
// * 1. .selection(red, blue);
// * 2. p {
// *        .selection(red, blue)
// *    }
// **/

.selection(@text-color, @background-color) {
	&::-moz-selection {color: @text-color; background-color: @background-color;}
	&::selection {color: @text-color; background-color: @background-color;}
}

///**
// * hasLayout
// * 
// * A hack for IE.
// *
// * @LINKS
// * * http://reference.sitepoint.com/css/haslayout
// **/

.hasLayout() {*zoom: 1;}


.ninja() {
	color: black;
	visibility: hidden;
}
