@import "vendor/3L.less";
@import "vendor/font-awesome/less/font-awesome.less";

@white: #f7f8f9;
@green: #29b96e;
@green2: #39ff98;
@green3: #0e4026;
@green4: #1c7f4c;
@gray: #333;
@gray2: #777;
@gray3: #999;
@border: #e3e3e3;
@weight1: 400;
@orange: #ff9f80;
@purple: #ad19ff;
@gold: #cc8c14;
@pink: #ef40ff;
@font1: "Open Sans", "Helvetica Neue", sans-serif;
@font2: "Droid Sans Mono", monospace, serif;

.normalize();

body {
    font-family: @font1;
    font-weight: @weight1;
    color: @gray;
    background: @white;
}

h1 {
    padding-bottom: 40px;
}

a:link, a:visited {
    color: @green;
    text-decoration: none;
}

a:hover, a:active {
    text-decoration: underline;
}

.border-round {
    border: 1px solid @border;
    .border-radius(6px);
}

* > a.headerlink {
    display: none;
}

dd {
    color: @gray3;
    font-weight: @weight1;
    max-width: 85%;
}

dl.function > dt, dl.class > dt, dl.method > dt {
    color: @gray;
    font-weight: bold;
}

dl tt {
    font-family: @font1;
}

dl.function, dl.class {
    background: #f3f4f5;
    padding: 10px;
    .border-round;
}

em.property {
    font-style: normal;
    color: @gray3;
}

dt > em {
    font-style: normal;
    font-weight: 400;
}

input {
    background: transparent;
    border: 1px solid @gray3;
    .border-radius(3px);
    padding: 2px 5px;
    color: @gray2;
    outline: none;
    &:focus {
        background: white;
    }
}

aside {
    position: fixed;
    top: 25px;
    width: 240px;
    color: @gray3;
    padding-left: 20px;

    &>ul {
        list-style: none;
        margin: 0 4px;
        padding: 0;
        &>li {
            margin-bottom: 10px;
            font-size: 18px;
            color: @gray2;
            a:link, a:visited {
                color: @gray2;
            }
            ul {
                margin: 10px 0 0 0;
                padding-left: 30px;
                font-size: 16px;
            }
        }
    }

    input[type=submit] {
        display: none;
    }

    input[type=text] {
        width: 85%;
    }
}

.literal {
    color: @green;
    font-size: 1em;
    padding: 1px 2px;
}

.admonition {
    padding: 10px 20px;
    border: 1px solid @border;
    .border-radius(6px);
}

.admonition-title {
    font-weight: 600;
}

.ad-core {
    border: 1px solid @border;
    .border-radius(6px);
    padding: 10px 20px 10px 70px;
    position: relative;
    color: white;
    margin: 10px 0px;
}

.ad-core-before {
    font-size: 2em;
    display: block;
    position: absolute;
    margin: auto;
    top: 0;
    bottom: 0;
    left: 20px;
    width: 32px;
    height: 32px;
}

.seealso
    {
    .ad-core;
    background: #e3e3e3;
    color: @gray;
    .admonition-title {display: none;}
    a { color: @green; }
    &:before
        {
        content: @fa-var-external-link;
        .fa;
        .ad-core-before;
        }
    .literal, .highlight-note
        {
        color: white;
        background: @green;
        padding: 1px 3px;
        .border-radius(2px);
        }
    }

.note
    {
    .ad-core;
    background: @green;
    .admonition-title {display: none;}
    a { color: #40ff85; }
    &:before
        {
        content: @fa-var-info-circle;
        .fa;
        .ad-core-before;
        }
    .literal, .highlight-note
        {
        color: white;
        background: #298354;
        padding: 1px 3px;
        .border-radius(2px);
        }
    }

.warning
    {
    .ad-core;
    background: #e0ab66;
    .admonition-title {display: none;}
    a { color: #633030; }
    &:before
        {
        content: @fa-var-warning;
        .fa;
        .ad-core-before;
        }
    .literal, .highlight-note
        {
        color: white;
        background: #9b6515;
        padding: 1px 3px;
        .border-radius(2px);
        }
    }

.danger
    {
    .ad-core;
    background: #d36871;
    .admonition-title {display: none;}
    a { color: #832929; }
    &:before
        {
        content: @fa-var-times-circle;
        .fa;
        .ad-core-before;
        }
    .literal, .highlight-note
        {
        color: white;
        background: #832929;
        padding: 1px 3px;
        .border-radius(2px);
        }
    }

.footnote {
    font-size: 0.9em;
    border: none;
}

table {
    border: 1px solid @border;
    border-color: @border;
}

table td, table th {
    padding: 5px;
}

table.field-list {
    border: none;
}

li.toctree-l1.current {
    font-weight: 700;
}

li.toctree-l2 {
    font-weight: @weight1;
}

span.viewcode-link {
    display: block;
    position: absolute;
    right: 0px;
    font-size: .8em;
    color: @green;
    padding-right: 10px;
}

div.document {
    max-width: 1000px;
    margin: 20px auto;
    position: relative;
    min-height: 550px;
}

div.documentwrapper {
    margin-left: 280px;
    padding: 0 0 0 20px;
}

div.body {
    padding-top: 10px;
}

div.footer {
    font-size: .8em;
    text-align: center;
    margin: 40px 0;
    color: @gray3;
}

.code-view {
    background: white;
    padding: 10px 20px;
    border: 1px solid @border;
    .border-radius(6px);
    font-family: @font2;
    font-size: 0.9em;
}

.highlight {
    background: transparent !important;

    pre {
        .code-view;
    }
}

.code {
    background: transparent !important;

    pre {
        .code-view;
    }
}

.highlight-python > pre {
    .code-view;
}

.pynb-result > pre {
    background: inherit;
    border: none;
}

img.align-center {
    display: block;
    margin: 0 auto;
    padding: 15px 0px;
}

img.pynb {
    background: #fff;
    padding: 20px;
    display: block;
    margin: 20px auto;
    max-width: 95%;
}

img.logo {
    display: block;
    margin: 0 auto;
    padding-bottom: 30px;
}

div.pynb-result {
    font-family: @font2;
    font-size: 0.9em;
    max-width: 98%;
    margin: 0 auto;
    
    table, p {
        margin-left: 20px;
    }
}

div#searchbox {
    padding-top: 10px;
}

@media (max-width: 800px) {
    div.documentwrapper {
        padding-top: 200px;
        margin: 0px 10px;
    }
    h1, .section {
        margin: 0px !important;
    }
    aside {
        position: static;
        width: 100%;
        margin: 5px 20px;
        padding: 5px;
    }
    #logo {
        position: absolute;
        top: 0px;
        left: 50%;
        margin-left: -75px;
    }
}
